﻿using System.Collections.Generic;
using Microsoft.SqlServer.Dts.Pipeline;
using Microsoft.SqlServer.Dts.Pipeline.Wrapper;
using Microsoft.WindowsAzure;

namespace TableStorageSsisSource
{
	[DtsPipelineComponent(DisplayName = "Azure Table Storage Source", ComponentType = ComponentType.SourceAdapter)]
	public class TableStorageSsisSource : PipelineComponent
	{
		public override void ProvideComponentProperties()
		{
			// Reset the component.
			base.RemoveAllInputsOutputsAndCustomProperties();
			ComponentMetaData.RuntimeConnectionCollection.RemoveAll();

			// Add output
			IDTSOutput100 output = ComponentMetaData.OutputCollection.New();
			output.Name = "Output";

			// Properties
			var storageConnectionStringProperty = this.ComponentMetaData.CustomPropertyCollection.New();
			storageConnectionStringProperty.Name = "StorageConnectionString";
			storageConnectionStringProperty.Description = "Azure storage connection string";
			storageConnectionStringProperty.Value = "UseDevelopmentStorage=true";

			var tableNameProperty = this.ComponentMetaData.CustomPropertyCollection.New();
			tableNameProperty.Name = "TableName";
			tableNameProperty.Description = "Name of the source table";
			tableNameProperty.Value = string.Empty;
		}

		public override IDTSCustomProperty100 SetComponentProperty(string propertyName, object propertyValue)
		{
			var resultingColumn = base.SetComponentProperty(propertyName, propertyValue);

			var storageConnectionString = (string)this.ComponentMetaData.CustomPropertyCollection["StorageConnectionString"].Value;
			var tableName = (string)this.ComponentMetaData.CustomPropertyCollection["TableName"].Value;

			if (!string.IsNullOrEmpty(storageConnectionString) && !string.IsNullOrEmpty(tableName))
			{
				var cloudStorageAccount = CloudStorageAccount.Parse(storageConnectionString);
				var context = new GenericTableContext(cloudStorageAccount.TableEndpoint.AbsoluteUri, cloudStorageAccount.Credentials);
				var firstRow = context.GetFirstOrDefault(tableName);
				if (firstRow != null)
				{
					var output = this.ComponentMetaData.OutputCollection[0];
					foreach (var column in firstRow.GetProperties())
					{
						var newOutputCol = output.OutputColumnCollection.New();
						newOutputCol.Name = column.ColumnName;
						newOutputCol.SetDataTypeProperties(column.DtsType, 0, 0, 0, 0);
					}
				}
			}

			return resultingColumn;
		}

		private List<ColumnInfo> columnInformation;
		private GenericTableContext context;
		private struct ColumnInfo
		{
			public int BufferColumnIndex;
			public string ColumnName;
		}

		public override void PreExecute()
		{
			this.columnInformation = new List<ColumnInfo>();
			IDTSOutput100 output = ComponentMetaData.OutputCollection[0];

			var cloudStorageAccount = CloudStorageAccount.Parse((string)this.ComponentMetaData.CustomPropertyCollection["StorageConnectionString"].Value);
			context = new GenericTableContext(cloudStorageAccount.TableEndpoint.AbsoluteUri, cloudStorageAccount.Credentials);

			foreach (IDTSOutputColumn100 col in output.OutputColumnCollection)
			{
				ColumnInfo ci = new ColumnInfo();
				ci.BufferColumnIndex = BufferManager.FindColumnByLineageID(output.Buffer, col.LineageID);
				ci.ColumnName = col.Name;
				columnInformation.Add(ci);
			}
		}

		public override void PrimeOutput(int outputs, int[] outputIDs, PipelineBuffer[] buffers)
		{
			IDTSOutput100 output = ComponentMetaData.OutputCollection[0];
			PipelineBuffer buffer = buffers[0];

			foreach (var item in this.context.CreateQuery<GenericEntity>((string)this.ComponentMetaData.CustomPropertyCollection["TableName"].Value))
			{
				buffer.AddRow();

				for (int x = 0; x < columnInformation.Count; x++)
				{
					var ci = (ColumnInfo)columnInformation[x];
					var value = item[ci.ColumnName].Value;
					if (value != null)
					{
						buffer[ci.BufferColumnIndex] = value;
					}
					else
					{
						buffer.SetNull(ci.BufferColumnIndex);
					}
				}
			}

			buffer.SetEndOfRowset();
		}
	}
}
